//*****************************************************************************
// Description     : debug only functions
// Target Device   : APG7400A
// Target Platform : Linux, C
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#ifndef     __MDEBUG_H__
#define     __MDEBUG_H__

#include    <time.h>
#include    <stdio.h>
#include    <stdlib.h>
#include    <string.h>
#include    <stdarg.h>
#include    <sys/time.h>

/* TLS: timestamp */
static __thread char    __d_stamp[512] = { 0 } ;


/* log output */
static void _d_dmp(int flag, char *fmt, ...)
{
    va_list     param ;
    
    if(flag != 0) {
        printf("%s ", &__d_stamp[1]) ;
    }
    
    va_start(param, fmt) ;
    vprintf(fmt, param) ;
    va_end(param) ;
    
    return ;
    
}

/* create timestamp and step */
static int __d_make_statmp(int flag, char *name, int nline)
{
    struct timeval      tmv ;
    struct timezone     tmz ;
    struct tm           ntm ;
    char    msg_time[48] ;
    
    if(flag == 0) {
        __d_stamp[0] = 0 ;
    }
    else {
        __d_stamp[0] = 1 ;
    }
    
    if(__d_stamp[0] == 1) {
        memset(&ntm, 0, sizeof(ntm)) ;
        memset(&tmv, 0, sizeof(tmv)) ;
        memset(&tmz, 0, sizeof(tmz)) ;
        
        gettimeofday(&tmv, &tmz) ;
        localtime_r(&tmv.tv_sec, &ntm) ;
        
        memset(msg_time, 0, sizeof(msg_time)) ;
        sprintf(msg_time, "%02d/%02d/%02d %02d:%02d:%02d.%03d",
                            (ntm.tm_year + 1900), (ntm.tm_mon + 1), ntm.tm_mday,
                             ntm.tm_hour, ntm.tm_min, ntm.tm_sec,
                            (int)(tmv.tv_usec / 1000)) ;
        sprintf(&__d_stamp[1], "[%s, (%4d) %s]: ", msg_time, nline, name) ;
    }
    
    return  (int)(__d_stamp[0]) ;
    
}

/* trace output */
#define     _D_TRACE_LFON   (1)
#define     _D_TRACE_LFOFF  (0)
#define     _d_trace(lf_off)      (lf_off != 0 ? _d_dmp(_d_on, "\n") : _d_dmp(_d_off, ""))

/* tiemstamp output */
#define     _d_on       __d_make_statmp(1, __FILE__, __LINE__)
#define     _d_off      __d_make_statmp(0, __FILE__, __LINE__)

#define     _d_trg(x)   {           \
    _d_on ;                         \
    _d_log("%s%s", &__d_stamp[1],   \
           (x != 0 ? "\n" : " ")) ; \
}


#define     _d_log      printf


/*
    using example
    (1) time and step output：     _d_dmp(_d_on,  "test1 = %d.\n", 1) ;
    (2) time and step non output： _d_dmp(_d_off, "test2 = %d.\n", 2) ;
*/


#endif

