//*****************************************************************************
// Description     : Definition of IO functions
// Target Device   : APG7400A
// Target Platform : Linux, C
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#include    <stdio.h>
#include    <stdlib.h>
#include    <string.h>

#include    "file.h"
#include    "usbmca.h"


/* #########################################################################
    Name      : file_read_binary
    Function  : write data to the text file
    Arguments :
        in name   : file name
        in nseek  : seek address
        in nbyte  : size of the data(byte)
        in buffer : pointer to the data buffer
    Return :
         0 : success
        -1 : failed to open file
        -2 : failed to seek
   ######################################################################### */
int file_read_binary(char *name, int nseek, int nbyte, void *buffer)
{
    FILE    *fp ;
    size_t  abyte ;
    int     ret ;
    int     result ;
    
    result = 0 ;
    
    fp = fopen(name, "rb") ;   /* ファイルオープン(read only)    */
    
    if(fp != NULL) {
        /* seek address */
        ret = fseek(fp, (long)(nseek), SEEK_SET) ;
        
        if(ret == 0) {  /* error.. */
            /* read file */
            abyte = fread(buffer, (size_t)(1), (size_t)(nbyte), fp) ;
            result = (int)(abyte) ;
        }
        else {          /* error move */
            result = -2 ;
        }
        
        fclose(fp) ;
    }
    else {
        result = -1 ;   /* error... */
    }
    
    return  result ;
    
}


/* #########################################################################
    Name      : file_write_binary
    Function  : write data to the binary file
    Arguments :
        in name   : file name
        in nseek  : seek address
        in nbyte  : size of the data(byte)
        in buffer : pointer to the data buffer
    Return :
         0 : success
        -1 : failed to open file
        -2 : failed to seek
   ######################################################################### */
int file_write_binary(char *name, int nseek, int nbyte, void *buffer)
{
    FILE    *fp ;
    size_t  abyte ;
    int     ret ;
    int     result ;
    
    result = 0 ;
    
    fp = fopen(name, "rb+") ;   /* file open */
    
    if(fp == NULL) {
        /* new create */
        fp = fp = fopen(name, "wb") ;
    }
    
    if(fp != NULL) {
        if(nseek >= 0) {
            /* move */
            ret = fseek(fp, (long)(nseek), SEEK_SET) ;
        }
        else {
            /* move to end of the file */
            ret = fseek(fp, (int)(0), SEEK_END) ;
        }
        
        if(ret == 0) {
            /* write  data */
            abyte  =  fwrite(buffer, (size_t)(1), (size_t)(nbyte), fp) ;
            result = (int)(abyte) ;
        }
        else {
            /* move error... */
            result = -2 ;
        }
        
        fclose(fp) ;
    }
    else {
        /* file open error... */
        result = -1 ;
    }
    
    return  result ;
    
}

/* #########################################################################
    Name      : file_write_histogram
    Function  : write data to the text file
    Arguments :
        in savefile : file name
        in info     : save data
    Return :
         0 : success
        -1 : failed to open file
   ######################################################################### */
int file_save_histogram(char *savefile, USBMCA_HISTOGRAM *info)
{
    FILE    *fp;
    int     i, j;
    int     result;
    char    text[1024], text0[256];

    result = 0;

    memset(text, 0, sizeof(text));
    memset(text0, 0, sizeof(text0));

    fp = fopen(savefile, "w");

    if (fp != NULL) {
        /* header save */
        fprintf(fp, "[header]\n");
        fprintf(fp, "meas time,%llu  #nsec\n", info->meas_time);

        /* total count */
        fprintf(fp, "total count");

        for (i = 0; i < (int)(_countof(info->totoal)); i++) {
            fprintf(fp, ",%d", info->totoal[i]);
        }

        fprintf(fp, "\n");

        /* adg gain */
        fprintf(fp, "ADG gain");

        for (i = 0; i < (int)(_countof(info->adg)); i++) {
            fprintf(fp, ",%d", info->adg[i]);
        }

        fprintf(fp, "\n\n");

        /* data(histogram) */
        fprintf(fp, "[data]\n");
        fprintf(fp, "#ch,CH1,CH2,CH3,CH4\n");

        /* data(histogram) save */
        for (i = 0; i < (int)(_countof(info->data[0])); i++) {
            sprintf(text, "%d", i);

            for (j = 0; j < (int)(_countof(info->data)); j++) {
                sprintf(text0, ",%d", info->data[j][i]);
                strcat(text, text0);
            }

            fprintf(fp, "%s\n", text);
        }

        fclose(fp);
    }
    else {
        result = -1;
    }


    return  result;

}

